              PROGRAM MESO
C$$$  MAIN PROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C MAIN PROGRAM:  MESO        MESOSCALE MODEL FORECAST DRIVER
C   PRGMMR: JANJIC           ORG: NP22        DATE: 99-01-20  
C
C ABSTRACT: MESO CONTAINS THE PRIMARY RUNSTREAM FOR THE NCEP MESOSCALE
C   FORECAST MODEL.  AFTER AN INITIAL CALL TO SUBROUTINE INIT, CALLS
C   ARE MADE TO SUBROUTINES WHICH COMPUTE THE VARIOUS DYNAMICAL AND
C   PHYSICAL PROCESSES IN THE MODEL.  THE VARIABLE 'NTSD' IS THE
C   FUNDAMENTAL TIMESTEP COUNTER AND THUS ITS VALUE DETERMINES WHEN
C   THE SUBROUTINES ARE CALLED.
C
C PROGRAM HISTORY LOG:
C   98-??-??  JANJIC      ORIGINATOR ON NONHYDROSTATIC COMPONENT
C                         AND OTHER VARIATIONS OF NCEP ETA MODEL
C   00-03-01  BLACK       RECODED FOR DISTRIBUTED MEMORY
C                         AND THREADING
C
C USAGE:  MAIN PROGRAM
C
C   INPUT FILES:  NONE (SEE SUBROUTINE INIT)
C
C   OUTPUT FILES:  NONE (SEE SUBROUTINE CHKOUT)
C
C   SUBPROGRAMS CALLED:
C     UNIQUE:
C       INIT     - INITIALIZE VARIABLES AT START OF INTEGRATION
C       PDTEDT   - UPDATE SURFACE PRESSURE TENDENCY AND ETADOT
C       ADVE     - TEST VERSION OF HORIZONTAL/VERTICAL ADVECTIONS
C       ADV2     - ADVECTION OF Q, CLOUD WATER, AND TKE
C       HDIFF    - LATERAL DIFFUSION
C       BOCOH    - UPDATE H POINTS ON THE BOUNDARIES
C       BOCOV    - UPDATE V POINTS ON THE BOUNDARIES
C       RADTN    - RADIATION DRIVER
C       RDTEMP   - APPLY TEMPERATURE TENDENCY DUE TO RADIATION
C       TURBL    - PERFORM THE VERTICAL TURBULENT EXCHANGE
C       SURFACE  - UPDATE SURFACE TEMPERATURE, MOISTURE, AND OTHER
C                  GROUND HYDROLOGY
C       GSCOND   - CLOUD WATER/ICE PHYSICS PARAMETERIZATION
C       CUCNVC   - CONVECTIVE ADJUSTMENT FOR DEEP OR SHALLOW CONVECTION
C       PRECPD   - GRID SCALE PRECIPITATION
C       VADZ     - VERTICAL ADVECTION OF HEIGHT
C       HADZ     - HORIZONTAL ADVECTION OF HEIGHT
C       EPS      - ADVECTION OF W
C       PFDHT    - PRESS GRAD FORCE, CORIOLIS
C       CHKOUT   - POST PROFILE DATA.  
C
C   EXIT STATES:
C     COND =   1 - NORMAL EXIT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 90
C   MACHINE : IBM SP
C
C$$$
C
C     ******************************************************************
C     *                                                                *
C     *       NONHYDROSTATIC LIMITED AREA NCEP MESO SCALE MODEL        *
C     *                                                                *
C     *                                                                *
C     *    NOAA / NATIONAL CENTERS FOR ENVIRONMENTAL PREDICTION,       *
C     *                                             CAMP SPRINGS, MD   *
C     *                                                                *
C     ******************************************************************
C
C     ******************************************************************
C     *                                                                *
C     *                        REFERENCES                              *
C     *             FOR THE DYNAMICAL PART OF THE MODEL                *
C     *                                                                *
C     *  PRESSURE GRADIENT FORCE, OMEGA-ALPHA TERM:                    *
C     *      JANJIC, Z.I., 1977:  PRESSURE GRADIENT FORCE AND ADVECTION*
C     *      SCHEME USED FOR FORECASTING WITH STEEP AND SMALL SCALE    *
C     *      TOPOGRAPHY.  CONTRIB. ATMOS. PHYS., 50, 186-199.          *
C     *                                                                *
C     *  SUBGRID COUPLING, TIME DIFFERENCING:                          *
C     *      JANJIC, Z.I., 1979:  FORWARD-BACKWARD SCHEME MODIFIED TO  *
C     *      PREVENT TWO-GRID-INTERVAL NOISE AND ITS APPLICATION IN    *
C     *      SIGMA COORDINATE MODELS.  CONTRIB. ATMOS. PHYS.,          *
C     *      52, 69-84.                                                *
C     *                                                                *
C     *  HORIZONTAL ADVECTION, CONTINUITY EQUATION:                    *
C     *      Z.I. JANJIC, 1984, MWR, 112, NO.6, 1234-1245.             *
C     *                                                                *
C     *  STEP-MOUNTAIN REPRESENTATION:                                 *
C     *      BRYAN, K., 1969: A NUMERICAL METHOD FOR THE STUDY OF THE  *
C     *      CIRCULATION OF THE WORLD OCEAN. J. COMP. PHYS, 4,         *
C     *      347-376.                                                  *
C     *                                                                *
C     *  INTERNAL BOUNDARIES, OMEGA-ALPHA TERM:                        *
C     *      MESINGER, F., Z.I. JANJIC, S. NICKOVIC, D. GAVRILOV AND   *
C     *      D.G. DEAVEN, 1988: THE STEP-MOUNTAIN COORDINATE: MODEL    *
C     *      DESCRIPTION AND PERFORMANCE FOR CASES OF ALPINE LEE       *
C     *      CYCLOGENESIS AND FOR A CASE OF AN APPALACHIAN             *
C     *      REDEVELOPMENT.  MON.WEA. REV., 116, 1493-1518.            *
C     *                                                                *
C     *  HYBRID PRESSURE-SIGMA COORDINATE:                             *
C     *      ARAKAWA, A. AND V. R. LAMB, 1977:  COMPUTATIONAL DESIGN   *
C     *      OF THE BASIC DYNAMICAL PROCESSES OF THE UCLA GENERAL      *
C     *      CIRCULATION MODEL.  METHODS IN COMPUTATIONAL PHYSICS, 17, *
C     *      ACADEMIC PRESS, 173-265.                                  *
C     *                                                                *
C     *  NONHYDROSTATIC DYNAMICS:                                      *
C     *      JANJIC, Z. I., J. P. GERRITY, JR. AND S. NICKOVIC, 2001:  *
C     *      AN ALTERNATIVE APPROACH TO NONHYDROSTATIC MODELING.       *
C     *      MONTHLY WEATHER REVIEW, 129, 1164-1178.                   *
C     *      JANJIC, Z. I., 2001:  A NONHYDROSTATIC MODEL BASED ON     * 
C     *      A NEW APPROACH.  METEOROLOGY AND ATMOSPHERIC PHYSICS,     *
C     *      IN PRINT.                                                 *
C     *                                                                *
C     *                     SOME  REFERENCES                           *
C     *             FOR THE PHYSICS PART OF THE MODEL                  *
C     *                                                                *
C     *  JANJIC, Z.I., 1990: THE STEP-MOUNTAIN COORDINATE:             *
C     *     PHYSICAL PACKAGE.  MONTHLY WEATHER REVIEW, VOL. 118,       *
C     *     NO. 7, 1429-1443.                                          *
C     *  JANJIC, Z.I., 1994: THE STEP MOUNTAIN ETA COORDINATE:         *
C     *     FURTHER DEVELOPMENTS OF THER CONVECTION, VISCOUS SUBLAYER, *
C     *     AND TURBULENCE CLOSURE SCHEMES.  MONTHLY WEATHER REVIEW,   *
C     *     VOL. 122, 927-945.                                         *
C     *                                                                *
C     *  ALSO SEE REFERENCES IN PHYSICAL SUBROUTINES                   *
C     *                                                                *
C     ******************************************************************
C
C
C     ******************************************************************
C     *                                                                *
C     *  THIS VERSION OF THE PROGRAM IS WRITTEN IN STANDARD ANSI       *
C     *  FORTRAN 90                                                    *
C     *                                                                *
C     *  ORIGINAL CODE DESIGN:  Z. JANJIC, UCAR VISITING SCIENTIST     *
C     *  PRINCIPAL PROGRAMMER:  T. BLACK, NCEP                         *
C     *                                                                *
C     ******************************************************************
C     *                                                                *
C     *  THE MODEL USES THE SEMI-STAGGERED E GRID IN ARAKAWA NOTATION. *
C     *  HORIZONTAL INDEXING IS TWO-DIMENSIONAL (BLACK).               *
C     *                                                                *
C     *                                                                *
C     *                                                                *
C     * H(1,JM)  V(1,JM)  H(2,JM)  V(2,JM) ...... V(IM-1,JM)  H(IM,JM) *
C     *    .        .        .        .               .          .     *
C     *    .        .        .        .               .          .     *
C     *    .        .        .        .               .          .     *
C     *    .        .        .        .               .          .     *
C     *                                                                *
C     * H(1,3)   V(1,3)   H(2,3)   V(2,3) ....... V(IM-1,3)   H(IM,3)  *
C     *                                                                *
C     * V(1,2)   H(1,2)   V(2,2)   H(2,2) ....... H(IM-1,2)   V(IM,2)  *
C     *                                                                *
C     * H(1,1)   V(1,1)   H(2,1)   V(2,1) ....... V(IM-1,1)   H(IM,1)  *
C     *                                                                *
C     *                                                                *
C     *                                                                *
C     *  ARRAYS ARE DIMENSIONED (IM,JM).  NOTE THAT A PHANTOM COLUMN   *
C     *  OF POINTS MUST EXIST ALONG THE EASTERN EDGE FOR THE ARRAYS    *
C     *  TO BE COMPLETE.                                               *
C     *                                                                *
C     *  THE TOTAL NUMBER OF GRID POINTS IN THE HORIZONTAL EXCLUDING   *
C     *  THE PHANTOM COLUMN IS IMJM=IM*JM-JM/2.                        *
C     *                                                                *
C     *  AUXILIARY ARRAYS ARE USED TO LOCATE NEIGHBORING GRID POINTS   *
C     *  WITH RESPECT TO A GIVEN GRID POINT.  IHE(J) IS THE INCREMENT  *
C     *  TO THE I INDEX NEEDED TO REFER TO THE V POINT EAST OF AN      *
C     *  H POINT THUS IHE(J)=0 ON ODD ROWS AND =1 ON EVEN ROWS.        *
C     *  IHW(J)=IHE(J)-1 IS THE INCREMENT TO THE INDEX OF AN H POINT   *
C     *  TO REFER TO THE V POINT TO THE WEST OF THAT H POINT.  THE     *
C     *  ANALOG EXISTS FOR THE ARRAYS IVE(J) AND IVW(J).               *
C     *                                                                *
C     *  BOUNDARY MASKS AND TOPOGRAPHY MASKS ARE DEFINED FOR VECTOR    *
C     *  PROCESSING. THE BOUNDARY MASKS HBM2(K) AND VBM2(K) ARE        *
C     *  EQUAL TO ONE EVERYWHERE EXCEPT AT THE TWO OUTERMOST ROWS      *
C     *  WHERE THEY ARE EQUAL TO ZERO. THE BOUNDARY MASK VBM3(K) IS    *
C     *  EQUAL TO ONE EVERYWHERE EXCEPT AT THE THREE OUTERMOST ROWS    *
C     *  WHERE IT IS EQUAL TO ZERO. THE TOPOGRAPHY MASKS (HTM(K,L)     *
C     *  AND VTM(K,L)) ARE SET TO ZERO UNDERNEATH THE TOPOGRAPHY AND   *
C     *  TO ONE ELSWHERE. IN ADDITION, FOR TREATMENT OF PHYSICAL       *
C     *  PROCESSES, MAXIMUM VALUES OF THE VERTICAL INDEX ARE DEFINED   *
C     *  AND STORED (LMH(K) AND LMV(K).
C     *                                                                *
C     ******************************************************************
      INCLUDE 'EXCHM.h'
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA,NEST
C-----------------------------------------------------------------------
      INCLUDE "parmeta"
      INCLUDE "mpif.h"
      INCLUDE "mpp.h"
#include "sp.h"
C-----------------------------------------------------------------------
      INCLUDE "CTLBLK.comm"
      INCLUDE "CONTIN.comm"
      INCLUDE "VRBLS.comm"
      INCLUDE "PVRBLS.comm"
      INCLUDE "NHYDRO.comm"
      INCLUDE "CLDWTR.comm"
C-----------------------------------------------------------------------
C***
C***  THE FOLLOWING ARE USED FOR TIMIMG PURPOSES ONLY
C***
      real*8 timef
      real nhb_tim,mpp_tim,init_tim
      common/timing/surfce_tim,nhb_tim,res_tim,exch_tim
CYL
      character envar*4, srfile*7
CYL
C-----------------------------------------------------------------------
C***
C***  INITIALIZE MPI,
C***  SETUP I/O SERVER MECHANICS AND CHECK FOR WHETHER A
C***  SUFFICIENT NUMBER OF MPI TASKS HAVE BEEN INITIATED.
C***  IF INSUFFICIENT MPI TASK HAVE BEEN INITIATED THE
C***  CODE WILL STOP IN SETUP_SERVERS
C***
      CALL SETUP_SERVERS(INPES*JNPES,
     *                   MYPE,
     *                   NPES,
     *                   IQUILT_GROUP,
     *                   INUMQ,
     *                   MPI_COMM_COMP,
     *                   MPI_COMM_INTER,
     *                   MPI_COMM_INTER_ARRAY)
C
      IF(MYPE.EQ.0)THEN
        CALL W3TAGB('ETAFCST ',0097,0365,0060,'NP22   ')
      ENDIF
C
      IF(MYPE.EQ.NPES)THEN
        CALL START()
      ENDIF
C
C***
C***  AT THIS POINT NPES IS THE NUMBER OF MPI TASKS WORKING ON THE
C***  MODEL INTEGRATION. ALL OTHER TASKS ARE I/O SERVERS.
C
C***  AND AWAY WE GO !
C***
       IF(MYPE.GE.NPES)THEN
C
C***  FIRE UP THE I/O SERVERS
C
         CALL QUILT
C
       ELSE
C***
C***  THESE ARE THE TASKS THAT DO THE MODEL INTEGRATION
C***
C-----------------------------------------------------------------------
      mpp_tim=   0.
C
      bocoh_tim= 0.
      bocov_tim= 0.
      chkout_tim=0.
      cltend_tim=0.
      cucnvc_tim=0.
      pfdht_tim= 0.
      exch_tim=  0.
      goss_tim=  0.
      gscond_tim=0.
      gsmdrive_tim=0.
      fltas_tim =0.
      hdiff_tim= 0.
      adve_tim=  0.
      vad2_tim=  0.
      had2_tim=  0.
      init_tim=  0.
      nhb_tim=   0.
      pdtedt_tim=0.
      radtn_tim= 0.
      rdtemp_tim=0.
      res_tim=   0.
      surfce_tim=0.
      turbl_tim= 0.
      vadz_tim=  0.
      hadz_tim=  0.
      eps_tim=   0.
      ddamp_tim= 0.
C-----------------------------------------------------------------------
C***
C***  INITIALIZE ALL QUANTITIES ASSOCIATED WITH GRID DECOMPOSITION
C***
      btimx=timef()
      btim=timef()
      CALL MPPINIT
      mpp_tim=mpp_tim+timef()-btim
C-----------------------------------------------------------------------
C--------INITIALIZE CONSTANTS AND VARIABLES-----------------------------
C--------DISTRIBUTE THE VALUES TO THE VARIOUS NODES/PEs-----------------
C-----------------------------------------------------------------------
      btim=timef()
      CALL INIT  
      init_tim=timef()-btim
C
      btim=timef()
      CALL GOSSIP
      goss_tim=goss_tim+timef()-btim
C-----------------------------------------------------------------------
C--------INVOKE THE LYNCH DIGITAL FILTER IF DESIRED--------------------
C-----------------------------------------------------------------------
      IF(NTSD.EQ.0)THEN
        btim=timef()
        DO NFLT=1,3
          IF(.NOT.NEST.AND.NFLT.GT.1.AND.MYPE.EQ.0)THEN
            REWIND NBC
            READ(NBC)
            READ(NBC)BCHR
          ENDIF
C
          CALL DIGFLT(NFLT)
C
        ENDDO
        digflt_tim=digflt_tim+timef()-btim
      ENDIF
C
C-----------------------------------------------------------------------
C Look at the environmental variable, tmmark, to see whether we are running
C fcst for Eta or EDAS (if Eta, we don't read in the ppt obs or call ADJPPT)
C-----------------------------------------------------------------------
C
      btim=timef()
      CALL GETENV("tmmark",ENVAR)
      IF(MYPE.EQ.0) PRINT *, "MESO finds that tmmark =",ENVAR
      IF(ENVAR.NE.'tm00') then
        write(srfile,10) envar
10      format('SR.',a4)
        open(90,file=srfile,form='unformatted')
        CALL READPCP
      endif
      fltas_tim=fltas_tim+timef()-btim
CYL
C-----------------------------------------------------------------------
C********ENTRY INTO THE TIME LOOP***************************************
C-----------------------------------------------------------------------
C
      GO TO 1200
C
 2000 CONTINUE
      NTSD=NTSD+1
      IF(MYPE.EQ.0)WRITE(6,2001) NTSD,(NTSD-1)*DT/3600.
      IF(MYPE.EQ.0)WRITE(0,2001) NTSD,(NTSD-1)*DT/3600.
 2001 FORMAT('MESO:  TIMESTEP NTSD=',I5,'  FCST TIME=',F7.3,' HOURS')
C-----------------------------------------------------------------------
C---------PRESS TEND, ETA/SIGMA DOT, VERT PART OF OMEGA-ALPHA-----------
C-----------------------------------------------------------------------
      btim=timef()
      call exch(pd,1,1,2)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL PDTEDT                      !Contains call to EXCH
      pdtedt_tim=pdtedt_tim+timef()-btim
C-----------------------------------------------------------------------
C--------------------ADVECTION OF T, U, AND V---------------------------
C-----------------------------------------------------------------------
      btim=timef()
      call exch(pdslo,1,4,4)
      call exch(t,lm,u,lm,v,lm,1,2)            !Exchange T, U, and V
C
C     THIS LINE REPLACE THE TWO LINES ABOVE WITH "CWAS"

      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL ADVE
      adve_tim=adve_tim+timef()-btim
C-----------------------------------------------------------------------
C-------------------VERTICAL ADVECTION OF HEIGHT------------------------
C-----------------------------------------------------------------------
      btim=timef()
      CALL VADZ
      vadz_tim=vadz_tim+timef()-btim
C-----------------------------------------------------------------------
C-------------------HORIZONTAL ADVECTION OF HEIGHT----------------------
C-----------------------------------------------------------------------
      btim=timef()
      call exch(u,lm,v,lm,1,2)            !Exchange U and V
      call exch(z,lm+1,1,2)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL HADZ
      hadz_tim=hadz_tim+timef()-btim
C-----------------------------------------------------------------------
C------------------------ ADVECTION OF W -------------------------------
C-----------------------------------------------------------------------
      btim=timef()
      call exch(w,lm+1,1,2)      !Exchange w
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL EPS 
      eps_tim=eps_tim+timef()-btim
C-----------------------------------------------------------------------
C------------------ADVECTION OF Q, TKE, AND CLOUD WATER-----------------
C-----------------------------------------------------------------------
          if(mod(ntsd-1,idtad).eq.0) then
      btim=timef()
      call vad2
      vad2_tim=vad2_tim+timef()-btim
          endif
!
          if(mod(ntsd-1,idtad).eq.0) then
      btim=timef()
      CALL EXCH(Q,LM,2,3)
      CALL EXCH(Q2,LM,2,3)
      CALL EXCH(CWM,LM,2,3)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      call had2                
      had2_tim=had2_tim+timef()-btim
          endif
C-----------------------------------------------------------------------
C------------------------RADIATION--------------------------------------
C-----------------------------------------------------------------------

      IF(MOD(NTSD-1,NRADS).EQ.0.OR.MOD(NTSD-1,NRADL).EQ.0)THEN
        btim=timef()
        CALL RADTN
        radtn_tim=radtn_tim+timef()-btim
      ENDIF
C-----------------------------------------------------------------------
C--------APPLY TEMPERATURE TENDENCY DUE TO RADIATION--------------------
C-----------------------------------------------------------------------
      btim=timef()
      CALL RDTEMP
      rdtemp_tim=rdtemp_tim+timef()-btim
C-----------------------------------------------------------------------
C--------TURBULENT PROCESSES AND PRECIPITATION--------------------------
C-----------------------------------------------------------------------
      adve_tim=adve_tim+timef()-btim
      IF(MOD(NTSD-1,NPHS).EQ.0)THEN
        btim=timef()
        CALL EXCH(PD,1,T,LM,1,1)       !Exchange PD, T
        CALL EXCH(Q,LM,1,1)            !Exchange Q
        CALL EXCH(CWM,LM,1,1)          !Exchange CWM

        CALL EXCH(UZ0,1,1,1)
        CALL EXCH(VZ0,1,1,1)
        exch_tim=exch_tim+timef()-btim
C
        btim=timef()
        CALL TURBL                     !Contains calls to EXCH
        turbl_tim=turbl_tim+timef()-btim
!-----------------------------------------------------------------------
!--- Store original temperature array
!-----------------------------------------------------------------------
        btim=timef()
        ICLTEND=-1
        CALL CLTEND (ICLTEND)
        cltend_tim=cltend_tim+timef()-btim
      ENDIF
C-----------------------------------------------------------------------
C--------CONVECTIVE PRECIPITATION---------------------------------------
C-----------------------------------------------------------------------
      IF(MOD(NTSD-1,NCNVC).EQ.0)THEN
        btim=timef()
        CALL CUCNVC
        cucnvc_tim=cucnvc_tim+timef()-btim
      ENDIF
C-----------------------------------------------------------------------
C-- GRIDSCALE MICROPHYSICS (CONDENSATION & PRECIPITATION; forecast only
C-----------------------------------------------------------------------
      IF(MOD(NTSD-1,NPHS).EQ.0)THEN
        btim=timef()
        CALL GSMDRIVE
        gsmdrive_tim=gsmdrive_tim+timef()-btim
!-----------------------------------------------------------------------
!-------- Calculate temp tendencies and restore original temps ---------
!-----------------------------------------------------------------------
        btim=timef()
        ICLTEND=0
        CALL CLTEND (ICLTEND)
        cltend_tim=cltend_tim+timef()-btim
      ENDIF
!-----------------------------------------------------------------------
!----- Update temp tendencies from cloud processes every time step -----
!-----------------------------------------------------------------------
      btim=timef()
      ICLTEND=1
      CALL CLTEND (ICLTEND)
      cltend_tim=cltend_tim+timef()-btim
C-----------------------------------------------------------------------
C--------LATERAL DIFFUSION----------------------------------------------
C-----------------------------------------------------------------------
      btim=timef()
      call exch(t,lm,q,lm,u,lm,v,lm,2,2)      !Exchange T, U, V, and Q
      CALL EXCH(Q2,LM,1,1)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL HDIFF
      hdiff_tim=hdiff_tim+timef()-btim
C-----------------------------------------------------------------------
C--------UPDATING BOUNDARY VALUES AT HEIGHT POINTS----------------------
C-----------------------------------------------------------------------
      btim=timef()
!        if(mod(ntsd,idtad).eq.0)then
      call exch(q,lm,1,1)
      call exch(cwm,lm,1,1)
      call exch(q2,lm,1,1)
!        endif
      call exch(pd,1,1,1)
      call exch(t,lm,1,1)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL BOCOH
      bocoh_tim=bocoh_tim+timef()-btim
C-----------------------------------------------------------------------
C--------IS IT TIME FOR A CHECK POINT ON THE MODEL HISTORY FILE?--------
C-----------------------------------------------------------------------
 1200 CONTINUE
C-----------------------------------------------------------------------
!!!!  IF(NTSD.EQ.0.OR.NTSD.GT.1.AND.NTSD-1.EQ.NSTART+1.
!!!! 1   OR.NTSD.GT.NSTART+1)THEN
        btim=timef()
        CALL CHKOUT 
        chkout_tim=chkout_tim+timef()-btim
!!!!  ENDIF
C
C-----------------------------------------------------------------------
C--------PRESS GRD, COR, DIV, AND HORIZ PART OF OMEGA-ALPHA ------------
C-----------------------------------------------------------------------
C
      IF(NTSD.GT.0)THEN
        btim=timef()
        CALL EXCH(PD,1,1,2)
      call exch(t,lm,u,lm,v,lm,q,lm,cwm,lm,dwdt,lm,pint,lm+1,1,2)
        exch_tim=exch_tim+timef()-btim
      ENDIF
C
      btim=timef()
      CALL PFDHT
      pfdht_tim=pfdht_tim+timef()-btim
C-----------------------------------------------------------------------
      btim=timef()
      call exch(div,lm,1,2)
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL DDAMP
      ddamp_tim=ddamp_tim+timef()-btim
C-----------------------------------------------------------------------
C--------CLEAN UP AFTER RESTART-----------------------------------------
C-----------------------------------------------------------------------
      IF(FIRST.AND.NTSD.EQ.0)THEN
        FIRST=.FALSE.
        GO TO 2000
      ENDIF
C-----------------------------------------------------------------------
C--------UPDATING BOUNDARY VALUES AT VELOCITY POINTS--------------------
C-----------------------------------------------------------------------
C
      btim=timef()
      CALL EXCH(U,LM,V,LM,1,1)           !Exchange U and V
      exch_tim=exch_tim+timef()-btim
C
      btim=timef()
      CALL BOCOV
      bocov_tim=bocov_tim+timef()-btim
C
C-----------------------------------------------------------------------
      IF(RESTRT)                           T H E N
        FIRST=.FALSE.
        RESTRT=.FALSE.
      ENDIF
C***********************************************************************
      IF(NTSD.LT.NTSTM)                 GO TO 2000
C***********************************************************************
C********EXIT FROM THE TIME LOOP****************************************
C***********************************************************************
C
 2005 continue
      tot2_tim=timef()-btimx
      tot_tim=mpp_tim+init_tim+goss_tim+fltas_tim+radtn_tim+chkout_tim+
     1 pdtedt_tim+adve_tim+vad2_tim+had2_tim+bocoh_tim+ddamp_tim+
     2 bocov_tim+rdtemp_tim+hdiff_tim+turbl_tim+
     3 cltend_tim+gsmdrive_tim+cucnvc_tim+
     4 vadz_tim+hadz_tim+eps_tim+pfdht_tim+exch_tim
C
      if(mype.eq.0)then
        pct=mpp_tim/tot_tim*1.e2
        write(6,*)' mpp=',mpp_tim*1.e-3,'    pct=',pct
        pct=init_tim/tot_tim*1.e2
        write(6,*)' init=',init_tim*1.e-3,'   pct=',pct
        pct=goss_tim/tot_tim*1.e2
        write(6,*)' goss=',goss_tim*1.e-3,'   pct=',pct
        pct=fltas_tim/tot_tim*1.e2
        write(6,*)' fltas=',fltas_tim*1.e-3,'   pct=',pct
        pct=chkout_tim/tot_tim*1.e2
        write(6,*)' chkout=',chkout_tim*1.e-3,' pct=',pct
        pct=pdtedt_tim/tot_tim*1.e2
        write(6,*)' pdtedt=',pdtedt_tim*1.e-3,'   pct=',pct
        pct=adve_tim/tot_tim*1.e2
        write(6,*)' adve=',adve_tim*1.e-3,'  pct=',pct
        pct=vad2_tim/tot_tim*1.e2
        write(6,*)' vad2=',vad2_tim*1.e-3,'  pct=',pct
        pct=had2_tim/tot_tim*1.e2
        write(6,*)' had2=',had2_tim*1.e-3,'  pct=',pct
        pct=radtn_tim/tot_tim*1.e2
        write(6,*)' radtn=',radtn_tim*1.e-3,'  pct=',pct
        pct=rdtemp_tim/tot_tim*1.e2
        write(6,*)' rdtemp=',rdtemp_tim*1.e-3,' pct=',pct
        pct=bocoh_tim/tot_tim*1.e2
        write(6,*)' bocoh=',bocoh_tim*1.e-3,'  pct=',pct
        pct=bocov_tim/tot_tim*1.e2
        write(6,*)' bocov=',bocov_tim*1.e-3,'  pct=',pct
        pct=hdiff_tim/tot_tim*1.e2
        write(6,*)' hdiff=',hdiff_tim*1.e-3,'  pct=',pct
        pct=ddamp_tim/tot_tim*1.e2
        write(6,*)' ddamp=',ddamp_tim*1.e-3,'  pct=',pct
        pct=turbl_tim/tot_tim*1.e2
        write(6,*)' turbl=',turbl_tim*1.e-3,'  pct=',pct
        pct=cltend_tim/tot_tim*1.e2
        write(6,*)' cltend=',cltend_tim*1.e-3,' pct=',pct
        pct=cucnvc_tim/tot_tim*1.e2
        write(6,*)' cucnvc=',cucnvc_tim*1.e-3,' pct=',pct
        pct=gsmdrive_tim/tot_tim*1.e2
        write(6,*)' gsmdrive=',gsmdrive_tim*1.e-3,' pct=',pct
        pct=vadz_tim/tot_tim*1.e2
        write(6,*)' vadz=',vadz_tim*1.e-3,'  pct=',pct
        pct=hadz_tim/tot_tim*1.e2
        write(6,*)' hadz=',hadz_tim*1.e-3,'  pct=',pct
        pct=eps_tim/tot_tim*1.e2
        write(6,*)' eps=',eps_tim*1.e-3,'  pct=',pct
        pct=pfdht_tim/tot_tim*1.e2
        write(6,*)' pfdht=',pfdht_tim*1.e-3,'  pct=',pct
        pct=exch_tim/tot_tim*1.e2
        write(6,*)' exch=',exch_tim*1.e-3,'   pct=',pct
        write(6,*)' total=',tot_tim*1.e-3
        write(6,*)' total2=',tot2_tim*1.e-3
      endif
C----------------------------------------------------------------------
C
C***  WE MUST NOW SHUT DOWN THE I/O SERVERS
C***  THIS IS DONE BY SENDING A -999 TO MPI TASK 0 OF EACH SERVER GROUP
C
      IF(MYPE.EQ.0)THEN
        DO I=1,IQUILT_GROUP
          CALL MPI_SEND(-999,1,MPI_INTEGER,0,0,
     *                  MPI_COMM_INTER_ARRAY(I),IER)
        ENDDO
      ENDIF
C
      IF ( MYPE .EQ. 0 )  THEN
         DO L = 1, LM
!            PRINT 100, L,q(15,20,L)
100         format(' L, q(15,20,L) = ',I5, Z8 )
         END DO
      END IF
C----------------------------------------------------------------------
C----------------------------------------------------------------------
      ENDIF    !  ENDIF ON TASKS FOR MODEL INTEGRATION VS I/O SERVING
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
      CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)
C
      IF(MYPE.EQ.0) THEN
        CALL W3TAGE('ETAFCST ')
      ENDIF
C
      IF(MYPE.EQ.NPES)THEN
        CALL SUMMARY()
      ENDIF
C
      CALL MPI_FINALIZE(IERR)
!

C----------------------------------------------------------------------
                            STOP1
                            END
