                              SUBROUTINE MOCON
C     ******************************************************************
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .
C SUBPROGRAM:    CUCNVC      CONVECTIVE PRECIPITATION PARAMETERIZATION
C   PRGRMMR: JANJIC          ORG: W/NP2      DATE: 93-11-02
C
C ABSTRACT:
C     CUCNVC CALCULATES THE EFFECTS OF THE DEEP AND SHALLOW SUB-GRID
C     SCALE CONVECTION FOLLOWING THE SCHEME DEVELOPED BY JANJIC (1994).
C     THIS SCHEME WAS DESIGNED STARTING FROM THE BETTS-MILLER SCHEME
C     (1986).  HOWEVER, SEVERAL CONCEPTUAL CHANGES OF THE BETTS-MILLER
C     SCHEME WERE INTRODUCED.  A NONDIMENSIONAL PARAMETER CALLED CLOUD
C     EFFICIENCY WAS DEFINED.  THIS PARAMETER DEPENDS ON THE DEEP
C     CONVECTIVE REGIME.  THE DEEP MOISTURE PROFILE AND THE RELAXATION
C     TIME ARE DETERMINED BY THE CLOUD EFFICIENCY.  IN ADDITION, THE
C     DEEP CONVECTION IS NOT ALLOWED IF THAT WOULD LEAD TO AN ENTROPY
C     DECREASE.  THE SHALLOW CLOUD TOP WAS REMOVED, AND THE SHALLOW
C     MOIST PROFILES ARE DEFINED FROM THE REQUIREMENT THAT SHALLOW
C     CONVECTION PRODUCE A POSITIVE ENTROPY CHANGE.
C
C PROGRAM HISTORY LOG:
C   87-09-??  JANJIC     - ORIGINATOR
C   90-11-21  JANJIC     - TWO SETS OF DSP PROFILES (FAST AND SLOW)
C                          REPLACE THE ORIGINAL ONE SET
C   95-03-25  BLACK      - CONVERSION FROM 1-D TO 2-D IN HORIZONTAL
C   96-03-28  BLACK      - ADDED EXTERNAL EDGE
C   98-11-02  BLACK      - MODIFIED FOR DISTRIBUTED MEMORY
C   99-??-??  JANJIC     - RH REPLACED DSP's 
C
C USAGE: CALL CUCNVC FROM MAIN PROGRAM EBU
C
C   INPUT ARGUMENT LIST:
C     NONE
C
C   OUTPUT ARGUMENT LIST:
C     NONE
C
C   OUTPUT FILES:
C     NONE
C
C   SUBPROGRAMS CALLED:
C
C     UNIQUE:
C        TTBLEX
C
C     LIBRARY:
C        NONE
C
C   COMMON BLOCKS: CTLBLK
C                  LOOPS
C                  MASKS
C                  PHYS
C                  VRBLS
C                  CNVCLD
C                  PVRBLS
C                  ACMCLH
C                  INDX
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 90
C   MACHINE : IBM SP
C$$$
C     ******************************************************************
C     *                                                                *
C     *  REFERENCES:                                                   *
C     *                                                                *
C     *  JANJIC, Z.I., 1994:  THE STEP-MOUNTAIN ETA COORDINATE MODEL:  *
C     *    FURTHER DEVELOPMENTS OF THE CONVECTION, VISCOUS SUBLAYER    *
C     *    AND TURBULENCE CLOSURE SCHEMES.  MONTHLY WEATHER REVIEW,    *
C     *    VOL. 122, 927-945.                                          *
C     *                                                                *
C     ******************************************************************
C----------------------------------------------------------------------- 
      INCLUDE "parmeta"
      INCLUDE "parm.tbl"
      INCLUDE "mpp.h"
C----------------------------------------------------------------------- 
      PARAMETER 
     &(G=9.806,CP=1004.6,CAPA=0.28589641) 
C----------------------------------------------------------------------- 
      PARAMETER      
     &(ELWV=2.50E6,ELIVW=2.72E6,ROW=1.E3,EPSQ=1.E-12 
     &,A2=17.2693882,A3=273.16,A4=35.86 
     &,TFRZ=273.15,T1=274.16,PQ0=379.90516 
     &,STABS=1.0,DTTOP=0.0 
     &,epsup=1.00,epsdn=1.05,epsth=0.0 

     &,rhlsc=0.50,rhhsc=0.99,stresh=0.90

     &,PBM=13000.,PQM=20000.,PNO=1000.,PONE=2500.,PSH=29000.
     &,PSHU=45000.)
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(efimn=.05,efmnt=.70,fcc=0.5,fcb=1.-fcc
     &,rhfs=.40,rhss=.90,rhfl=.40,rhsl=.90
!zj     &,rhfc0=0.90,rhfct=.35,fct=0.95
!zj     &,rhfc0=0.95,rhfct=.50,fct=0.95
     &,rhfc0=0.95,rhfct=.95,fct=0.95
     &,trel=2400.,stabd=0.90,epsntp=.0001,efifc=2.0
     &,stefi=1.,avgefi=(efimn+1.)*.5,epsp=1.e-7 
     &,SLOPS=(RHFS-RHSS)/(1.-EFIMN) 
     &,SLOPL=(RHFL-RHSL)/(1.-EFIMN) 
     &,SLOPE=(1.-EFMNT)/(1.-EFIMN))
C
                             P A R A M E T E R 
     &(A23M4L=A2*(A3-A4)*ELWV) 
C
                             P A R A M E T E R 
     &(ELOCP=ELIVW/CP,CPRLG=CP/(ROW*G*ELWV),RCP=1./CP) 
C---------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(JAM=6+2*(JM-10),LP1=LM+1,LM1=LM-1,KSMUD=0,IMJM_LOC=IDIM2*JDIM2)
C---------------------------------------------------------------------- 
                             L O G I C A L 
     & RUN,FIRST,RESTRT,SIGMA 
C---------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PHYS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CNVCLD.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
C-----------------------------------------------------------------------
      INCLUDE "ACMCLH.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C----------------------------------------------------------------------- 
                             D I M E N S I O N
     & TREFK (LM),QREFK (LM),RHK   (LM),PK    (LM),APEK  (LM),TK    (LM) 
     &,QK    (LM),THERK (LM) 
     &,THVREF(LM),DIFT  (LM),DIFQ  (LM) 
     &,QSATK (LM),FPK   (LM) 
     &,NTOPD (LM),NBOTD (LM),NTOPS (LM),NBOTS (LM) 
     &,NDPTHD(LM),NDPTHS(LM) 
C 
                             D I M E N S I O N
     & LTOP  (IDIM1:IDIM2,JDIM1:JDIM2),LBOT  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PTOP  (IDIM1:IDIM2,JDIM1:JDIM2),PBOT  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,IPTB  (IDIM1:IDIM2,JDIM1:JDIM2),ITHTB (IDIM1:IDIM2,JDIM1:JDIM2)
     &,RH    (IDIM1:IDIM2,JDIM1:JDIM2),PDSL  (IDIM1:IDIM2,JDIM1:JDIM2) 
     &,PP    (IDIM1:IDIM2,JDIM1:JDIM2),QQ    (IDIM1:IDIM2,JDIM1:JDIM2) 
     &,PSP   (IDIM1:IDIM2,JDIM1:JDIM2),THBT  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,P     (IDIM1:IDIM2,JDIM1:JDIM2),THESP (IDIM1:IDIM2,JDIM1:JDIM2) 
     &,PRECOL(IDIM1:IDIM2,JDIM1:JDIM2) 
C
     &,ISHAL (IMJM_LOC),JSHAL (IMJM_LOC)
     &,IDEEP (IMJM_LOC),JDEEP (IMJM_LOC)
     &,ILRES (IMJM_LOC),JLRES (IMJM_LOC)
     &,IHRES (IMJM_LOC),JHRES (IMJM_LOC)
C
     &,APE   (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,TREF  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,TMOD  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,QMOD  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
C 
                             D I M E N S I O N
     & TL    (IDIM1:IDIM2,JDIM1:JDIM2),QL    (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TNE   (IDIM1:IDIM2,JDIM1:JDIM2),TSE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,QNE   (IDIM1:IDIM2,JDIM1:JDIM2),QSE   (IDIM1:IDIM2,JDIM1:JDIM2)
C----------------------------------------------------------------------- 
      AVCNVC=AVCNVC+1. 
      ACUTIM=ACUTIM+1. 
      DTCNVC=NCNVC*DT 
      RDTCNVC=1./DTCNVC 
      TAUK=DTCNVC/TREL 
      TAUKSC=DTCNVC/(1.*TREL) 
C----------------------------------------------------------------------- 
      IF(NTSD.LE.NPHS)THEN 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
!zj          CLDEFI(I,J)=AVGEFI*SM(I,J)+STEFI*(1.-SM(I,J)) 
          cldefi(i,j)=avgefi
        ENDDO 
        ENDDO 
      ENDIF 
C-----------------------------------------------------------------------
C************************** DIAGNOSTICS ********************************
C-----------------------------------------------------------------------
      DO L=1,LM
        NTOPD (L)=0
        NBOTD (L)=0
        NTOPS (L)=0
        NBOTS (L)=0
        NDPTHS(L)=0
        NDPTHD(L)=0
      ENDDO
C*********************************************************************** 
C--------------PREPARATIONS--------------------------------------------- 
C-----------------------------------------------------------------------
!$omp parallel do
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        LBOT (I,J)=LMH(I,J) 
        THESP(I,J)=0. 
        PDSL (I,J)=RES(I,J)*PD(I,J) 
        PRECOL(I,J)=0. 
        TREF(I,J,1)=T(I,J,1) 
      ENDDO 
      ENDDO 
C-----------------------------------------------------------------------
C--------------PADDING SPECIFIC HUMIDITY IF TOO SMALL-------------------
C                  RESTORE APE TO SCRATCH ARRAY
C-----------------------------------------------------------------------
!$omp parallel do private(apests)
      DO 130 L=1,LM
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        apests=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt
        APE(I,J,L)=(1.E5/APESTS)**CAPA
        IF(Q(I,J,L).LT.EPSQ)Q(I,J,L)=HTM(I,J,L)*EPSQ
      ENDDO
      ENDDO
  130 CONTINUE
C-----------------------------------------------------------------------
C--------------SEARCH FOR MAXIMUM BUOYANCY LEVEL------------------------
C-----------------------------------------------------------------------
                             DO 170 KB=1,LM
C-----------------------------------------------------------------------
C--------------TRIAL MAXIMUM BUOYANCY LEVEL VARIABLES-------------------
C-----------------------------------------------------------------------
!$omp parallel do
!$omp&  private(apesp,bq,bqs00k,bqs10k,iq,it,ittb,ittbk,iqtb,
!$omp&          lmhk,p00k,p01k,p10k,p11k,pkl,pp1,psfck,qbt,qq1,
!$omp&          sq,sqs00k,sqs10k,tpsp,tq,tth,tthbt,tthes)
      DO 155 J=MYJS2,MYJE2
      DO 150 I=MYIS1,MYIE1
C
      pkl=aeta1(kb)*pdtop+aeta2(kb)*pdsl(i,j)+pt
      LMHK=LMH(I,J)
      psfck=aeta1(lmhk)*pdtop+aeta2(lmhk)*pdsl(i,j)+pt
c     IF(PKL.LT.PSFCK-PBM)GO TO 150
c     IF(KB.GT.LMHK)GO TO 150
C
      IF(PKL.GE.PSFCK*(1.-PBM/100000.).AND.KB.LE.LMHK)THEN
!zj      IF(PKL.GE.PSFCK*0.6.AND.KB.LE.LMHK)THEN
        QBT=Q(I,J,KB)
        TTHBT=T(I,J,KB)*APE(I,J,KB)
        TTH=(TTHBT-THL)*RDTH
        QQ1=TTH-AINT(TTH)
        ITTB=INT(TTH)+1
C--------------KEEPING INDICES WITHIN THE TABLE-------------------------
        IF(ITTB.LT.1)THEN
          ITTB=1
          QQ1=0.
        ENDIF
C
        IF(ITTB.GE.JTB)THEN
          ITTB=JTB-1
          QQ1=0.
        ENDIF
C--------------BASE AND SCALING FACTOR FOR SPEC. HUMIDITY---------------
        ITTBK=ITTB
        BQS00K=QS0(ITTBK)
        SQS00K=SQS(ITTBK)
        BQS10K=QS0(ITTBK+1)
        SQS10K=SQS(ITTBK+1)
C--------------SCALING SPEC. HUMIDITY & TABLE INDEX---------------------
        BQ  =(BQS10K-BQS00K)*QQ1+BQS00K
        SQ  =(SQS10K-SQS00K)*QQ1+SQS00K
        TQ  =(QBT-BQ)/SQ*RDQ
        PP1 =TQ - AINT(TQ)
        IQTB=INT(TQ)+1
C--------------KEEPING INDICES WITHIN THE TABLE-------------------------
        IF(IQTB.LT.1)THEN
          IQTB=1
          PP1=0.
        ENDIF
C
        IF(IQTB.GE.ITB)THEN
          IQTB=ITB-1
          PP1=0.
        ENDIF
C--------------SATURATION PRESSURE AT FOUR SURROUNDING TABLE PTS.-------
        IQ=IQTB
        IT=ITTB
        P00K=PTBL(IQ  ,IT  )
        P10K=PTBL(IQ+1,IT  )
        P01K=PTBL(IQ  ,IT+1)
        P11K=PTBL(IQ+1,IT+1)
C--------------SATURATION POINT VARIABLES AT THE BOTTOM-----------------
        TPSP=P00K+(P10K-P00K)*PP1+(P01K-P00K)*QQ1
     1           +(P00K-P10K-P01K+P11K)*PP1*QQ1
        APESP=(1.E5/TPSP)**CAPA
        TTHES=TTHBT*EXP(ELOCP*QBT*APESP/TTHBT)
C--------------CHECK FOR MAXIMUM BUOYANCY-------------------------------
        IF(TTHES.GT.THESP(I,J))THEN
          PSP  (I,J)=TPSP
          THBT (I,J)=TTHBT
          THESP(I,J)=TTHES
        ENDIF
      ENDIF
C
  150 CONTINUE
  155 CONTINUE
C-----------------------------------------------------------------------
  170 CONTINUE
C-----------------------------------------------------------------------
C---------CHOOSE CLOUD BASE AS MODEL LEVEL JUST BELOW PSP--------------
C-----------------------------------------------------------------------
      DO 240 L=1,LM1
!$omp parallel do
      DO J=MYJS2,MYJE2
      DO I=MYIS,MYIE
        p(i,j)=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt
        IF(P(I,J).LT.PSP(I,J).AND.P(I,J).GE.PQM)LBOT(I,J)=L+1
      ENDDO
      ENDDO
C
  240 CONTINUE
C***
C*** WARNING: LBOT MUST NOT BE GT LMH(I,J)-1 IN SHALLOW CONVECTION
C*** MAKE SURE CLOUD BASE IS AT LEAST PONE ABOVE THE SURFACE
C***
!$omp parallel do private(lmhij,psfck)
      DO 250 J=MYJS2,MYJE2
      DO 250 I=MYIS,MYIE
      LMHIJ=LMH(I,J)
      pbot(i,j)=aeta1(lbot(i,j))*pdtop+aeta2(lbot(i,j))*pdsl(i,j)+pt
      psfck=aeta1(lmhij)*pdtop+aeta2(lmhij)*pdsl(i,j)+pt
      IF(PBOT(I,J).GE.PSFCK-PONE.OR.LBOT(I,J).GE.LMHIJ)THEN
C
        DO L=1,LMHIJ-1
          p(i,j)=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt
          IF(P(I,J).LT.PSFCK-PONE)LBOT(I,J)=L
        ENDDO
C
        pbot(i,j)=aeta1(lbot(i,j))*pdtop+aeta2(lbot(i,j))*pdsl(i,j)+pt
      ENDIF
  250 CONTINUE
C--------------CLOUD TOP COMPUTATION------------------------------------
!$omp parallel do
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
!zj
        deep(i,j)=0.
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
!$omp parallel do
!$omp&  private(ihres,ilres,iptb,ithtb,jhres,jlres,
!$omp&          knumh,knuml,pp,presk,qq)
C
      DO 290 L=LM,1,-1
C
C--------------SCALING PRESSURE & TT TABLE INDEX------------------------
      KNUML=0
      KNUMH=0
C
      DO 270 J=MYJS2,MYJE2
      DO 270 I=MYIS1,MYIE1
      presk=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt
      IF(PRESK.LT.PLQ)THEN
        KNUML=KNUML+1
        ILRES(KNUML)=I
        JLRES(KNUML)=J
      ELSE
        KNUMH=KNUMH+1
        IHRES(KNUMH)=I
        JHRES(KNUMH)=J
      ENDIF
 270  CONTINUE
C***
C***  COMPUTE PARCEL TEMPERATURE ALONG MOIST ADIABAT FOR PRESSURE<PL
C**
      IF(KNUML.GT.0)THEN
        call ttblex(tref(idim1,jdim1,l),ttbl,itb,jtb,knuml
     1,             ilres,jlres
     &,             pdtop,pdsl,aeta1(l),aeta2(l),htm(idim1,jdim1,l)
     2,             pt,pl,qq(idim1,jdim1),pp(idim1,jdim1)
     3,             rdp,the0,sthe,rdthe
     4,             thesp(idim1,jdim1),iptb(idim1,jdim1)
     5,             ithtb(idim1,jdim1))
      ENDIF
C***
C***  COMPUTE PARCEL TEMPERATURE ALONG MOIST ADIABAT FOR PRESSURE>PL
C**
      IF(KNUMH.GT.0)THEN
        call ttblex(tref(idim1,jdim1,l),ttblq,itbq,jtbq,knumh
     1,             ihres,jhres
     &,             pdtop,pdsl,aeta1(l),aeta2(l),htm(idim1,jdim1,l)
     2,             pt,plq,qq(idim1,jdim1),pp(idim1,jdim1)
     3,             rdpq,the0q,stheq,rdtheq
     4,             thesp(idim1,jdim1),iptb(idim1,jdim1)
     5,             ithtb(idim1,jdim1))
      ENDIF
 290  CONTINUE
C--------------BUOYANCY CHECK-------------------------------------------
      DO 295 L=LM,1,-1
!$omp parallel do
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
!zj      ltop(i,j)=lm
!zj      ptpk=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt
!zj      if(tref(i,j,l).gt.t(i,j,l)-dttop.and.ptpk.gt.pqm) ltop(i,j)=l
        IF(TREF(I,J,L).GT.T(I,J,L)-DTTOP)LTOP(I,J)=L
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
  295 CONTINUE
C-----------------CLOUD TOP PRESSURE------------------------------------
!$omp parallel do
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
        ptop(i,j)=aeta1(ltop(i,j))*pdtop+aeta2(ltop(i,j))*pdsl(i,j)+pt
      ENDDO
      ENDDO
C--------------DEFINE RELATIVE HUMIDITY--------------------------------- 
!$omp parallel do
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        EFI=CLDEFI(I,J) 
        RH(I,J)=((EFI-EFIMN)*SLOPS+RHSS)*SM(I,J) 
     1         +((EFI-EFIMN)*SLOPL+RHSL)*(1.-SM(I,J)) 
      ENDDO 
      ENDDO 
C--------------INITIALIZE CHANGES OF T AND Q DUE TO CONVECTION----------
!$omp parallel do
      DO 360 L=1,LM
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        TMOD(I,J,L)=0.
        QMOD(I,J,L)=0.
      ENDDO
      ENDDO
  360 CONTINUE
C--------------CLEAN UP AND GATHER DEEP CONVECTION POINTS---------------
!$omp parallel do
      DO 380 J=MYJS2,MYJE2
      DO 380 I=MYIS1,MYIE1
      IF(LTOP(I,J).GE.LBOT(I,J))THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
      ENDIF
      IF(HBM2(I,J).LT.0.90)THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
      ENDIF
      IF(PTOP(I,J).GT.PBOT(I,J)-PNO.OR.LTOP(I,J).GT.LBOT(I,J)-2)
!zj     1 CLDEFI(I,J)=AVGEFI*SM(I,J)+STEFI*(1.-SM(I,J))
     & cldefi(i,j)=avgefi
  380 CONTINUE
C
      KHDEEP=0
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
        IF(PTOP(I,J).LT.PBOT(I,J)-PSH)THEN
          KHDEEP=KHDEEP+1
          IDEEP(KHDEEP)=I
          JDEEP(KHDEEP)=J
        ENDIF
        ENDDO
      ENDDO
C************* HORIZONTAL LOOP FOR DEEP CONVECTION *********************
!$omp parallel do
!$omp&  private(apek,avrgt,avtgtl,dentpy,dhdt,difq,difql,
!$omp&          dift,diftl,drheat,drhdp,dp,efi,fefi,hcorr,
!$omp&          i,j,l0,l0m1,lb,lbm1,lbtk,lcor,lqm,ltp1,ltp2,
!$omp&          ltpk,ltsh,pbtk,pk,pk0,pkb,pkl,pkt,preck,
!$omp&          pthrs,ptpk,qk,qkl,qrefk,qsatk,qstop,
!$omp&          RH0,rhbot,rhh,rhk,rhl,rhmax,rhtop,
!$omp&          sumde,sumdp,therk,tk,tkl,toprh,trefk,trefkl)
C***********************************************************************
      DO 600 N=1,KHDEEP
C***********************************************************************
      I=IDEEP(N)
      J=JDEEP(N)
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDC  DEEP CONVECTION   DCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
      LTPK=LTOP(I,J)
      LBTK=LBOT(I,J)
      LB   =LBTK
      EFI  =CLDEFI(I,J)
C--------------INITIALIZE VARIABLES IN THE CONVECTIVE COLUMN------------ 
      DO L=1,LM 
        DIFT(L)=0. 
        DIFQ(L)=0. 
C 
        TKL=T(I,J,L) 
        TK(L)=TKL 
        TREFK(L)=TKL 
C 
        QKL=Q(I,J,L) 
        QK(L)=QKL 
        QREFK(L)=QKL 
C 
        pk(l)=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt 
        apekl=ape(i,j,l) 
        apek(l)=apekl
C 
        therk(l)=tref(i,j,l)*apekl 
        RHK(L)=0. 
      ENDDO 
C-------------DEEP CONVECTION REFERENCE TEMPERATURE PROFILE------------- 
      LTP1=LTPK+1 
      LBM1=LB-1 
C 
      PKB=PK(LB) 
      PKT=PK(LTPK) 
!-------------temperature reference profile below freezing level--------
      l0=lb
      pk0=pk(lb)
      trefdn=trefk(lb)
      apekdn=apek(lb)
      therdn=therk(lb)
      apekup=apek(lbm1)
      therup=therk(lbm1)
          do l=ltpk,lbm1
      ivi=ltpk+lbm1-l
      if (t(i,j,ivi+1).lt.273.16) go to 430
!      
      trefdn=((therup-therdn)*stabd+trefdn*apekdn)/apekup
      trefk(ivi)=trefdn
      apekdn=apekup
      therdn=therup
      apekup=apek(ivi-1)
      therup=therk(ivi-1)
      l0=ivi
      pk0=pk(l0)
          enddo 
!-------------freezing level at or above the cloud top------------------
      l0m1=l0-1
      go to 450
!-------------temperature reference profile above freezing level--------
 430  continue
      l0m1=l0-1
      rdp0t=1./(pk0-pkt)
      dthem=therk(l0)-trefk(l0)*apek(l0)
          do l=ltpk,l0m1
      trefk(l)=(therk(l)-(pk(l)-pkt)*dthem*rdp0t)/apek(l)
          enddo 
 450  continue                
!-------------RELATIVE HUMIDITY PROFILE--------------------------------- 
      rhbot=rh(i,j) 
      rhtop=rhbot*rhfct
C----------------------------------------------------------------------- 
      IF(L0.GE.LTP1.AND.L0.LE.LBM1)THEN 
C----------------------------------------------------------------------- 
        rh0=rhbot*rhfc0
C 
        RHSLOP=(RH0-RHBOT)/(PK0-PKB) 
        DO L=L0,LB 
          RHK(L)=(PK(L)-PKB)*RHSLOP+RHBOT 
        ENDDO 
C
        RHSLOP=(RHTOP-RH0)/(PKT-PK0)       
        DO L=LTPK,L0-1 
          RHK(L)=(PK(L)-PK0)*RHSLOP+RH0 
        ENDDO 
C----------------------------------------------------------------------- 
      ELSE 
C----------------------------------------------------------------------- 
        RHSLOP=(RHTOP-RHBOT)/(PKT-PKB) 
        DO L=LTPK,LB 
          RHK(L)=(PK(L)-PKB)*RHSLOP+RHBOT 
        ENDDO 
C----------------------------------------------------------------------- 
      ENDIF 
!-------------DEEP CONVECTION REFERENCE HUMIDITY PROFILE---------------- 
      DO L=LTPK,LB 
        IF(PK(L).GT.PQM)THEN 
          qrefk(l)=rhk(l)*pq0/pk(l)*exp(a2*(trefk(l)-a3) 
     1                                    /(trefk(l)-a4)) 
        ELSE 
          QREFK(L)=Q(I,J,L) 
        ENDIF 
      ENDDO 
!      if(mype.eq.3.and.i.eq.32.and.j.eq.32.and.ntsd.gt.300) then
!          print*,'tk=',tk
!          print*,'qk=',qk
!          print*,'therk=',therk
!      endif
C--------------ENTHALPY CONSERVATION INTEGRAL-------------------------- 
      DO ITER=1,2 
C----------------------------------------------------------------------- 
        SUMDE=0. 
        SUMDP=0. 
C
        DO L=LTPK,LB
          dp=deta1(l)*pdtop+deta2(l)*pdsl(i,j) 
          sumde=((tk(l)-trefk(l))*cp+(qk(l)-qrefk(l))*elwv)*dp
     1          +sumde 
          sumdp=sumdp+dp 
        ENDDO 
C
        hcorr=sumde/(sumdp-(deta1(ltpk)*pdtop+deta2(ltpk)*pdsl(i,j))) 
        LCOR=LTPK+1 
!-----------------------FIND LQM---------------------------------------- 
        LQM=LM 
        DO L=1,LB 
          IF(PK(L).LE.PQM)LQM=L 
        ENDDO 
!--------------ABOVE LQM CORRECT TEMPERATURE ONLY----------------------- 
        IF(LCOR.LE.LQM)THEN 
          DO L=LCOR,LQM 
            TREFK(L)=TREFK(L)+HCORR*RCP 
          ENDDO 
          LCOR=LQM+1 
        ENDIF 
!-------------BELOW LQM CORRECT BOTH TEMPERATURE AND MOISTURE----------- 
        DO L=LCOR,LB 
          TREFKL=TREFK(L) 
          DHDT=QREFK(L)*A23M4L/(TREFKL-A4)**2+CP 
          TREFKL=HCORR/DHDT+TREFKL 
      if(trefkl.lt.180.)write(0,*)i,j,l,trefkl,qrefk(l),dhdt
          QREFK(L)=RHK(L)*PQ0/PK(L)*EXP(A2*(TREFKL-A3)/(TREFKL-A4)) 
          TREFK(L)=TREFKL 
        ENDDO 
C----------------------------------------------------------------------- 
      ENDDO     ! End of Iteration Loop
!      if(mype.eq.3.and.i.eq.32.and.j.eq.32.and.ntsd.gt.300) then
!          print*,'trefk=',trefk
!          print*,'qrefk=',qrefk
!      endif
C----------------------------------------------------------------------- 
C--------------HEATING, MOISTENING, PRECIPITATION----------------------- 
C----------------------------------------------------------------------- 
      DENTPY=0. 
      AVRGT=0. 
      PRECK=0. 
C 
      DO L=LTPK,LB 
        dp=deta1(l)*pdtop+deta2(l)*pdsl(i,j)
        TKL=TK(L) 
        DIFTL=(TREFK(L)-TKL  )*TAUK 
        DIFQL=(QREFK(L)-QK(L))*TAUK 
        AVRGTL=(TKL+TKL+DIFTL) 
        dentpy=(diftl*cp+difql*elwv)*dp/avrgtl+dentpy 
        avrgt=avrgtl*dp+avrgt 
        preck=diftl*dp+preck 
        DIFT(L)=DIFTL 
        DIFQ(L)=DIFQL 
      ENDDO 
C
      DENTPY=DENTPY+DENTPY 
      AVRGT=AVRGT/(SUMDP+SUMDP) 
C
      drheat=preck*cp/avrgt
      EFI=EFIFC*DENTPY/DRHEAT
      EFI=CLDEFI(I,J)*FCB+EFI*FCC
C-------------SWAP IF ENTROPY AND/OR PRECIP .LT. 0 ...------------------
      if(dentpy.lt.epsntp.or.preck.lt.0..or.efi.lt.efimn)then
C-------------REINITIALIZE CLOUD EFFICIENCY----------------------------- 
!zj        CLDEFI(I,J)=EFIMN*SM(I,J)+STEFI*(1.-SM(I,J)) 
        cldefi(i,j)=avgefi
C--------------SEARCH FOR SHALLOW CLOUD TOP----------------------------- 
        LBTK=LBOT(I,J) 
        LTSH=LBTK 
        LBM1=LBTK-1 
        PBTK=PK(LBTK) 
        PTPK=PBTK-PSH 
C-------------CLOUD TOP IS THE LEVEL JUST BELOW PBTK-PSH---------------- 
        DO L=1,LM 
          IF(PK(L).LE.PTPK)LTPK=L+1 
        ENDDO 
        PTPK=PK(LTPK) 
C--------------HIGHEST LEVEL ALLOWED IS LEVEL JUST BELOW PSHU----------- 
        IF(PTPK.LE.PSHU)THEN 
          DO L=1,LM 
            IF(PK(L).LE.PSHU)LTPK=L+1 
          ENDDO 
          PTPK=PK(LTPK) 
        ENDIF 
C----------------------------------------------------------------------- 
        LTP1=LTPK+1 
        LTP2=LTPK+2 
C----------------------------------------------------------------------- 
        DO L=LTPK,LBTK 
          QSATK(L)=PQ0/PK(L)*EXP(A2*(TK(L)-A3)/(TK(L)-A4)) 
        ENDDO 
C----------------------------------------------------------------------- 
        RHH=QK(LTPK)/QSATK(LTPK) 
        RHMAX=0. 
C
        DO L=LTP1,LBM1 
          RHL=QK(L)/QSATK(L) 
          DRHDP=(RHH-RHL)/(PK(L-1)-PK(L)) 
          IF(DRHDP.GT.RHMAX)THEN 
            LTSH=L-1 
            RHMAX=DRHDP 
          ENDIF 
          RHH=RHL 
        ENDDO 
C 
        LTOP(I,J)=LTSH 
C---------------CLOUD MUST BE AT LEAST TWO LAYERS THICK----------------- 
        IF(LBOT(I,J)-LTOP(I,J).LT.2)LTOP(I,J)=LBOT(I,J)-2 
C 
        PTOP(I,J)=PK(LTOP(I,J)) 
C----------------------------------------------------------------------- 
      ELSE 
C----------------------------------------------------------------------- 
C----------------- DEEP CONVECTION OTHERWISE --------------------------- 
C----------------------------------------------------------------------- 
        EFI=MIN(EFI,1.) 
        IF(PRECK.EQ.0.)EFI=1. 
        CLDEFI(I,J)=EFI 
!zj
        deep(i,j)=1.
C 
        FEFI=EFMNT+SLOPE*(EFI-EFIMN) 
C 
        PRECK=PRECK*FEFI 
C 
C--------------UPDATE PRECIPITATION, TEMPERATURE & MOISTURE------------- 
        precol(i,j)=preck*cprlg 
        prec(i,j)=preck*cprlg+prec(i,j) 
        cuprec(i,j)=preck*cprlg+cuprec(i,j) 
        acprec(i,j)=preck*cprlg+acprec(i,j) 
        cuppt(i,j)=preck*cprlg+cuppt(i,j) 
!      if(mype.eq.3.and.i.eq.32.and.j.eq.32.and.ntsd.gt.300) then
!          print*,'dentpy=',dentpy,' drheat=',drheat,' preck=',preck
!          print*,'avrgt=',avrgt,'efi=',efi,' fefi=',fefi
!          print*,' fis=',fis(i,j)
!          print*,'dift=',dift
!          print*,'difq=',difq
!      endif
C
        DO L=LTPK,LB 
          TMOD(I,J,L)=DIFT(L)*FEFI 
          QMOD(I,J,L)=DIFQ(L)*FEFI 
        ENDDO 
C
C----------------------------------------------------------------------- 
      ENDIF 
C----------------------------------------------------------------------- 
C
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDC          END OF DEEP CONVECTION            DCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
C
C-----------------------------------------------------------------------
  600 CONTINUE
C----------------------------------------------------------------------- 
      NDEEP=0
C
      DO 620 J=MYJS2,MYJE2
      DO 620 I=MYIS,MYIE
      LTPK=LTOP(I,J)
      LBTK=LBOT(I,J)
      LB  =LMH(I,J)-1
      IF(PTOP(I,J).LT.PBOT(I,J)-PSH)THEN
        NDEEP=NDEEP+1
        NDEPTH=LBTK-LTPK
        NTOPD (LTPK  )=NTOPD (LTPK  )+1
        NBOTD (LBTK  )=NBOTD (LBTK  )+1
        NDPTHD(NDEPTH)=NDPTHD(NDEPTH)+1
      ENDIF
  620 CONTINUE
      NNEG=KHDEEP-NDEEP
C
C--------------GATHER SHALLOW CONVECTION POINTS------------------------- 
C
      KHSHAL=0 
      NDSTN=0 
      NDSTP=0 
C
              DO J=MYJS2,MYJE2
          DO I=MYIS1,MYIE1

!zj              if(lpbl(i,j).ge.lbot(i,j) then

          if(lpbl(i,j).lt.lbot(i,j)) then
      lbot(i,j)=lpbl(i,j) 
          endif 
      if(lbot(i,j).gt.lmh(i,j)-1) lbot(i,j)=lmh(i,j)-1 
! 
      pbot(i,j)=aeta1(lbot(i,j))*pdtop+aeta2(lbot(i,j))*pdsl(i,j)+pt 
! 
              if(.not.(ptop(i,j).gt.pbot(i,j)-pno.or. 
     &           ltop(i,j).le.lbot(i,j)-2)) then 
          if(ptop(i,j)+1..ge.pbot(i,j)-psh) then 
      khshal=khshal+1 
      ishal(khshal)=i 
      jshal(khshal)=j 
          endif 
              endif 

!zj                  endif

          ENDDO 
              ENDDO 
C
C************* HORIZONTAL LOOP FOR SHALLOW CONVECTION ******************
!$omp parallel do
!$omp&  private(apek,apekl,apekdn,apekup,bqk,bqs00k,bqs10k,den,dentpy,
!$omp&           dp,dpkl,dpmix,dqref,dst,dstq,dtdeta,fpk,fptk,i,iq,it,j,
!$omp&           lbm1,lbtk,ltp1,ltpk,otsum,part1,part2,part3,pk,pkl,
!$omp&           pkxxxx,pkxxxy,potsum,ppk,psum,ptpk,qk,qkl,qnew,
!$omp&           qotsum,qqk,qrefk,qrfkl,qrftp,qsatk,qsum,rdpsum,rhnew,
!$omp&           rtbar,smix,sqk,sqs00k,sqs10k,sumdp,sumdt,tcorr,
!$omp&           thvmkl,thvref,tk,tkl,tqk,trefk,trefdn,trfkl,tthk)
C
      DO 800 N=1,KHSHAL
C***********************************************************************
      I=ISHAL(N)
      J=JSHAL(N)
C-----------------------------------------------------------------------
CSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCS
CSCSCSCSCSCSC         SHALLOW CONVECTION          CSCSCSCSCSCSCSCSCSCSCS
CSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCS
C-----------------------------------------------------------------------
      lmp=lmh(i,j)
!
      do l=1,lmp 
        TKL=T(I,J,L) 
        TK(L)=TKL 
        TREFK(L)=TKL 
        QKL=Q(I,J,L) 
        QK(L)=QKL 
        QREFK(L)=QKL 
        pkl=aeta1(l)*pdtop+aeta2(l)*pdsl(i,j)+pt 
        PK(L)=PKL 
        QSATK(L)=PQ0/PK(L)*EXP(A2*(TK(L)-A3)/(TK(L)-A4)) 
        APEKL=APE(I,J,L) 
Cvvvvvvvvvvvv choose the pressure function vvvvvvvvvvvvvvvvvvvvvvvvvvvvv 
c         FPK  (L) =ALOG(PKL) 
c         FPK  (L) =PKL 
c         FPK  (L) =-1./PKL 
Caaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa 
        APEK(L)=APEKL 
        THVMKL=TKL*APEKL*(QKL*.608+1.) 
        THVREF(L)=THVMKL 
      ENDDO 
C--------------SHALLOW CLOUD TOP---------------------------------------- 
      LBTK=LBOT(I,J) 
      LBM1=LBTK-1 
      PTPK=PTOP(I,J) 
      LTP1=LTOP(I,J) 
      LTPK=LTOP(I,J)-1 
C----------------------------------------------------------------------- 
      IF(PTOP(I,J).GT.PBOT(I,J)-PNO.OR.LTOP(I,J).GT.LBOT(I,J)-2)THEN 
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J) 
        PTOP(I,J)=PBOT(I,J) 
        GO TO 800 
      ENDIF 
C
C--------------SCALING POTENTIAL TEMPERATURE & TABLE INDEX AT TOP------- 
C
c     THTPK=T(I,J,LTP1)*APE(I,J,LTP1) 
      THTPK=T(I,J,LTPK)*APE(I,J,LTPK) 
C 
      TTHK=(THTPK-THL)*RDTH 
      QQK=TTHK-AINT(TTHK) 
      IT=INT(TTHK)+1 
C
      IF(IT.LT.1)THEN 
        IT=1 
        QQK=0. 
      ENDIF 
C
      IF(IT.GE.JTB)THEN 
        IT=JTB-1 
        QQK=0. 
      ENDIF 
C--------------BASE AND SCALING FACTOR FOR SPEC. HUMIDITY AT TOP-------- 
      BQS00K=QS0(IT)
      SQS00K=SQS(IT)
      BQS10K=QS0(IT+1)
      SQS10K=SQS(IT+1)
C--------------SCALING SPEC. HUMIDITY & TABLE INDEX AT TOP--------------
      BQK=(BQS10K-BQS00K)*QQK+BQS00K
      SQK=(SQS10K-SQS00K)*QQK+SQS00K
C
C     TQK=(Q(I,J,LTP1)-BQK)/SQK*RDQ
      TQK=(Q(I,J,LTPK)-BQK)/SQK*RDQ
C
      PPK=TQK-AINT(TQK)
      IQ =INT(TQK)+1
      IF(IQ.LT.1)THEN
        IQ=1
        PPK=0.
      ENDIF
      IF(IQ.GE.ITB)THEN
        IQ=ITB-1
        PPK=0.
      ENDIF
C--------------CLOUD TOP SATURATION POINT PRESSURE----------------------
      PART1=(PTBL(IQ+1,IT)-PTBL(IQ,IT))*PPK
      PART2=(PTBL(IQ,IT+1)-PTBL(IQ,IT))*QQK
      PART3=(PTBL(IQ  ,IT  )-PTBL(IQ+1,IT  )
     1      -PTBL(IQ  ,IT+1)+PTBL(IQ+1,IT+1))*PPK*QQK
      PTPK=PTBL(IQ,IT)+PART1+PART2+PART3
C-----------------------------------------------------------------------
      DPMIX=PTPK-PSP(I,J)
      IF(ABS(DPMIX).LT.3000.)DPMIX=-3000.
C--------------TEMPERATURE PROFILE SLOPE--------------------------------
      SMIX=(THTPK-THBT(I,J))/DPMIX*STABS
C
      TREFKX=TREFK(LBTK+1)
      PKXXXX=PK(LBTK+1)
      PKXXXY=PK(LBTK)
      APEKXX=APEK(LBTK+1)
      APEKXY=APEK(LBTK)
C
      DO 670 L=LBTK,LTP1,-1
      TREFKX=((PKXXXY-PKXXXX)*SMIX
     1        +TREFKX*APEKXX)/APEKXY
      TREFK(L)=TREFKX
      APEKXX=APEKXY
      PKXXXX=PKXXXY
      APEKXY=APEK(L-1)
      PKXXXY=PK(L-1)
  670 CONTINUE
C--------------TEMPERATURE REFERENCE PROFILE CORRECTION-----------------
      SUMDT=0.
      SUMDP=0.
C
      DO L=LTP1,LBTK
        dp=deta1(l)*pdtop+deta2(l)*pdsl(i,j)
        sumdt=(tk(l)-trefk(l))*dp+sumdt
        sumdp=sumdp+dp
      ENDDO
C
      RDPSUM=1./SUMDP
      FPK(LBTK)=TREFK(LBTK)
C
      TCORR=SUMDT*RDPSUM
C
      DO L=LTP1,LBTK
        TRFKL   =TREFK(L)+TCORR
        TREFK(L)=TRFKL
        FPK  (L)=TRFKL
      ENDDO
C--------------HUMIDITY PROFILE EQUATIONS-------------------------------
      PSUM  =0.
      QSUM  =0.
      POTSUM=0.
      QOTSUM=0.
      OTSUM =0.
      DST   =0.
      FPTK  =FPK(LTP1)
C
      DO 700 L=LTP1,LBTK
      dp=deta1(l)*pdtop+deta2(l)*pdsl(i,j)
      DPKL  =FPK(L)-FPTK
      psum  =dpkl*dp+psum
      qsum  =qk(l)*dp+qsum
      RTBAR =2./(TREFK(L)+TK(L))
      otsum =rtbar*dp+otsum
      potsum=dpkl*rtbar*dp+potsum
      qotsum=qk(l)*rtbar*dp+qotsum
      dst   =(trefk(l)-tk(l))*rtbar*dp+dst
  700 CONTINUE
C
      PSUM  =PSUM*RDPSUM
      QSUM  =QSUM*RDPSUM
      ROTSUM=1./OTSUM
      POTSUM=POTSUM*ROTSUM
      QOTSUM=QOTSUM*ROTSUM
      DST   =DST   *ROTSUM*CP/ELWV
C--------------ENSURE POSITIVE ENTROPY CHANGE---------------------------
      IF(DST.GT.0.)THEN
C
C       DSTQ=DST*EPSUP
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
cccc    NDSTP=NDSTP+1
        GO TO 800
C
      ELSE
        DSTQ=DST*EPSDN
      ENDIF
C--------------CHECK FOR ISOTHERMAL ATMOSPHERE--------------------------
      DEN=POTSUM-PSUM
C
      IF(-DEN/PSUM.LT.5.E-5)THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
        GO TO 800
C
C--------------SLOPE OF THE REFERENCE HUMIDITY PROFILE------------------
      ELSE
        DQREF=(QOTSUM-DSTQ-QSUM)/DEN
      ENDIF
C------------ HUMIDITY DOES NOT INCREASE WITH HEIGHT--------------------
      IF(DQREF.LT.0.)THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
        GO TO 800
      ENDIF
C--------------HUMIDITY AT THE CLOUD TOP--------------------------------
      QRFTP=QSUM-DQREF*PSUM
C--------------HUMIDITY PROFILE-----------------------------------------
C
      DO 720 L=LTP1,LBTK
      QRFKL=(FPK(L)-FPTK)*DQREF+QRFTP
C
C--------------SUPERSATURATION OR NEGATIVE Q NOT ALLOWED----------------
C
      QNEW =(QRFKL-QK(L))*TAUKSC+QK(L)
      IF(QNEW.LT.QSATK(L)*RHLSC.OR.QNEW.GT.QSATK(L)*RHHSC)THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
        GO TO 800
      ENDIF
C----------------------------------------------------------------------- 
      THVREF(L)=TREFK(L)*APEK(L)*(QRFKL*.608+1.) 
      QREFK(L)=QRFKL 
  720 CONTINUE
C
C--------------TOO DRY CLOUD BOTTOMS NOT ALLOWED------------------------ 
C
      RHNEW=((QREFK(LBTK)-QK(LBTK))*TAUKSC+QK(LBTK))/QSATK(LBTK) 
      IF(RHNEW.LT.QK(LBTK+1)/QSATK(LBTK+1)*STRESH)THEN 
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J) 
        PTOP(I,J)=PBOT(I,J) 
        GO TO 800 
      ENDIF 
C--------------ELIMINATE IMPOSSIBLE SLOPES (BETTS, DTHETA/DQ)-----------
      DO 730 L=LTP1,LBTK
      dtdeta=(thvref(l-1)-thvref(l))
     &      /((aeta1(l)-aeta1(l-1))*pdtop
     &       +(aeta2(l)-aeta2(l-1))*pdsl(i,j))
      IF(DTDETA.LT.EPSTH)THEN
        LBOT(I,J)=0
        LTOP(I,J)=LBOT(I,J)
        PTOP(I,J)=PBOT(I,J)
        GO TO 800
      ENDIF
  730 CONTINUE
C*************************** DIAGNOSTICS ******************************
cccc  IF(DST.GT.0.)THEN
cccc    NDSTP=NDSTP+1
cccc  ELSE
cccc    NDSTN=NDSTN+1
cccc  ENDIF
C**********************************************************************
      DENTPY=0.
C
      DO L=LTP1,LBTK
        dentpy=((trefk(l)-tk(l))*cp+(qrefk(l)-qk(l))*elwv)
     1         /(tk(l)+trefk(l))
     &        *(deta1(l)*pdtop+deta2(l)*pdsl(i,j))
     &        +dentpy
      ENDDO
C
C-----------------------------------------------------------------------
C--------------RELAXATION TOWARDS REFERENCE PROFILES--------------------
C-----------------------------------------------------------------------
C
C
      DO 750 L=LTP1,LBTK
      TMOD(I,J,L)=(TREFK(L)-TK(L))*TAUKSC
      QMOD(I,J,L)=(QREFK(L)-QK(L))*TAUKSC
  750 CONTINUE
C-----------------------------------------------------------------------
CSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCS
CSCSCSCSCSCSC         END OF SHALLOW CONVECTION        SCSCSCSCSCSCSCSCS
CSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCSCS
C-----------------------------------------------------------------------
  800 CONTINUE
C----------------------------------------------------------------------- 
C
C************************** DIAGNOSTICS ********************************
      NSHAL=0
C
      DO 820 J=MYJS2,MYJE2
      DO 820 I=MYIS,MYIE
      LTPK=LTOP(I,J)
      LBTK=LBOT(I,J)
      PTPK=PTOP(I,J)
      PBTK=PBOT(I,J)
      IF(PTPK.GT.PBTK-PNO.OR.LTPK.GT.LBTK-2)GO TO 820
      IF(PTPK.GE.PBTK-PSH)THEN
        NSHAL=NSHAL+1
        NTOPS(LTPK)=NTOPS(LTPK)+1
        NBOTS(LBTK)=NBOTS(LBTK)+1
        NDEPTH=LBTK-LTPK
        NDPTHS(NDEPTH)=NDPTHS(NDEPTH)+1
      ENDIF
  820 CONTINUE
      NEGDS=KHSHAL-NSHAL
C***********************************************************************
C
C--------------SMOOTHING TEMPERATURE & HUMIDITY CORRECTIONS-------------
      IF(KSMUD.EQ.0)THEN
!$omp parallel do
C
        DO 900 L=1,LM
C***
C***  UPDATE THE FUNDAMENTAL PROGNOSTIC ARRAYS
C***
        DO 830 J=MYJS,MYJE
        DO 830 I=MYIS,MYIE
        T(I,J,L)=T(I,J,L)+TMOD(I,J,L)
        Q(I,J,L)=Q(I,J,L)+QMOD(I,J,L)
C***
C***  ACCUMULATE LATENT HEATING DUE TO CONVECTION.
C***  SCALE BY THE RECIPROCAL OF THE PERIOD AT WHICH THIS ROUTINE
C***  IS CALLED.  THIS PERIOD IS THE CONVECTION TIMESTEP.
C***
        TCUCN(I,J,L)=TCUCN(I,J,L)+TMOD(I,J,L)*RDTCNVC
 830    CONTINUE
 900    CONTINUE
      ELSE
C***
!$omp parallel do private(ql,qne,qse,tl,tne,tse)
        DO 910 L=1,LM
C
        CALL ZERO2(QL)
        CALL ZERO2(QNE)
        CALL ZERO2(QSE)
        CALL ZERO2(TL)
        CALL ZERO2(TNE)
        CALL ZERO2(TSE)
C
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          TL(I,J)=TMOD(I,J,L)
          QL(I,J)=QMOD(I,J,L)
        ENDDO
        ENDDO
C-----------------------------------------------------------------------
        NSMUD=KSMUD
C
        DO 870 KS=1,NSMUD
C
        DO J=MYJS,MYJE1
        DO I=MYIS,MYIE
        TNE(I,J)=(TL(I+IHE(J),J+1)-TL(I,J))
     1           *HTM(I,J,L)*HTM(I+IHE(J),J+1,L)
        QNE(I,J)=(QL(I+IHE(J),J+1)-QL(I,J))
     1           *HTM(I,J,L)*HTM(I+IHE(J),J+1,L)
        ENDDO
        ENDDO
C
        DO J=MYJS1,MYJE
        DO I=MYIS,MYIE
        TSE(I,J)=(TL(I+IHE(J),J-1)-TL(I,J))
     1           *HTM(I+IHE(J),J-1,L)*HTM(I,J,L)
        QSE(I,J)=(QL(I+IHE(J),J-1)-QL(I,J))
     1           *HTM(I+IHE(J),J-1,L)*HTM(I,J,L)
        ENDDO
        ENDDO
C
        DO J=MYJS2,MYJE2
        DO I=MYIS,MYIE
        TL(I,J)=(TNE(I,J)-TNE(I+IHW(J),J-1)+TSE(I,J)-TSE(I+IHW(J),J+1))
     1          *HBM2(I,J)*D125+TL(I,J)
        QL(I,J)=(QNE(I,J)-QNE(I+IHW(J),J-1)+QSE(I,J)-QSE(I+IHW(J),J+1))
     1          *HBM2(I,J)*D125+QL(I,J)
        ENDDO
        ENDDO
C
  870   CONTINUE
C-----------------------------------------------------------------------
C***
C***  UPDATE THE FUNDAMENTAL PROGNOSTIC ARRAYS
C***
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          T(I,J,L)=T(I,J,L)+TL(I,J)
          Q(I,J,L)=Q(I,J,L)+QL(I,J)
        ENDDO
        ENDDO
C***
C***  ACCUMULATE LATENT HEATING DUE TO CONVECTION.
C***  SCALE BY THE RECIPROCAL OF THE PERIOD AT WHICH THIS ROUTINE
C***  IS CALLED.  THIS PERIOD IS THE CONVECTION TIMESTEP.
C***
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          TCUCN(I,J,L)=TCUCN(I,J,L)+TL(I,J)*RDTCNVC
        ENDDO
        ENDDO
C
  910   CONTINUE
C
      ENDIF
C--------------SAVE CLOUD TOP AND BOTTOM FOR RADIATION------------------
!$omp parallel do
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        HTOP(I,J)=MIN(FLOAT(LTOP(I,J)),HTOP(I,J))
        HBOT(I,J)=MAX(FLOAT(LBOT(I,J)),HBOT(I,J))
        CNVTOP(I,J)=MIN(FLOAT(LTOP(I,J)),CNVTOP(I,J))
        CNVBOT(I,J)=MAX(FLOAT(LBOT(I,J)),CNVBOT(I,J))
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
C************************* DIAGNOSTICS *********************************
C
!zj          if(mype.eq.0.) then
!zj      WRITE(LIST,950) NTSD,NSHAL,NDEEP,NNEG,NEGDS,NDSTN,NDSTP
!zj          DO 940 L=1,LM
!zj      WRITE(LIST,952)L
!zj      WRITE(LIST,954)NBOTS(L),NTOPS(L),NDPTHS(L)
!zj     1,              NBOTD(L),NTOPD(L),NDPTHD(L)
!zj 940  CONTINUE
 950  FORMAT(' NTSD=',I3,I8,' SHALLOW, ',I4,' DEEP, ',
     1 I4,' NEG., ',I4,' NEG. SHALL.,',I4,' DST.LT.0, ',I4,' DST.GT.0')
 952  FORMAT(' LAYER (FROM TOP),',I2)
 954  FORMAT('     NBOTS=',I4,'     NTOPS=',I4,'     NDPTHS=',I4,
     1       '     NBOTD=',I4,'     NTOPD=',I4,'     NDPTHD=',I4)
!zj         endif
C***********************************************************************
                             RETURN
                             END
